#include "Gadget/Engine.h" //This file includes all the other engine files

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	//Here I'll create an instance of "Core" and call it "G". You can of course name it anything you like, 
	//but keeping the name short is important because we will end up typing it a lot. If you type G followed by 
	//a period, you will see a drop down list of all the stuff the engine can do, well not all of it. There are 
	//several other support classes provided to load graphics, play sounds, and other stuff.
	Core G; //Create an instance of the game engine

	G.Window.SetClearColor(0.7f, 0.5f, 0.3f); //Set the background color to green

	G.Window.Open(); //This line of code creates a window.

	G.Window.SetTitle("Graphics & Sound"); //Here I have changed the name of the window to "Graphics & Sound"
	//You can change the window's name as often as you like. This can be used for debugging.

	/*
	FMOD is a commercial sound engine used in many of the games you play. Its a full featured sound engine, but to make 
	things simple I made a wrapper class that reduces its functionality somewhat. I have kept it separate from the rest 
	of the code in case I want to replace it with another sound engine later.
	*/
	SoundEngine S; //Create an instance of the sound engine, Only make one!

	Sound beep; //Create a sound
	//The load function needs the file name of the sound and the address of the sound engine
	beep.Load("Sounds/Beeps.wav", &S, false, false); 
	
	/*
	The Image class loads almost every image file type, jpg, gif(no animation), png, tga, and many more. It uses the 
	DevIL API which is free and cross platform.
	Note: The width and height of your image should be a power of 2. Many video cards perform poorly for odd sized graphics. 
	Examples: 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048
	*/
	Image buttonImg;
	buttonImg.Load("Images/ClickHere.png"); //Load the image from disk.

	/*
	A Sprite is an object that we apply images and shaders to. You can think of it as a sign, and the image is what gets painted 
	on it. The sign can be moved, resized, and rotated. Here I created a Sprite and initialized its width and height.  Notice 
	that the width and height do not have to match the image width and height. When the size does not match the image, the image 
	will be stretched to fit.
	*/
	Sprite buttonSpr(250.0f, 100.0f); 
	buttonSpr.x = 400.0f; //The horizontal position on screen (by default a window is 800 pixels wide so I have placed it in the center)
	buttonSpr.y = 300.0f; //The vertical position on screen (by default, a window is 600 pixels high)

	while(G.Window.IsOpen()) //Loop continuously as long as the engine is running
	{
		G.Renderer.BeginFrame(); //Clears the buffers
		G.Renderer.Render2D();   //Render a 2D scene

			/*
			You can check to see if a key is being pressed at any time using the standard key codes or by using characters. For 
			example, I could check to see if the letter B is pressed like this G.Keyboard.GetKeyPress(B).
			*/
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	//ESC key closes the window
			
			if(G.Mouse.GetLeftPress())//Check to see if the left mouse button has been pressed
			{
				//If the mouse point is over the button play a sound
				if(buttonSpr.PointOnSprite(G.Mouse.GetPosition()))
				{
					beep.Play();
				}
			}

			//Even though we are not using any shader effects here, my shader class also has some blending options.
			//The png image has transparent corners, but without blending they would be rendered opaque. The supported 
			//blend modes are; 
			// (0) Opague - Ignores image alpha
			// (1) Threshold - Each pixel is either completely transparent or completely opague
			// (2) Image Alpha - Uses the image's alpha channel
			// (3) Additive - The RGB values are used to brighten the underlying scene (appears to glow)
			// (4) Subtractive - The RGB values are used to darken the underlying scene
			Shader::Blend(2);

			buttonImg.Activate(); //Only one material can be active at one time
			buttonSpr.Render();   //Draw the sprite with the active material painted on it

			//Shader effects and blending will affect everything you render! So it's a good idea to turn them off 
			//when you're finished with them. In fact, some older video cards will run really slow if shaders are
			//active when the screen refreshes. It's a good idea to turn them off when you are finished rendering
			//your scene.
			Shader::Deactivate(); //Turns off blending and any active shader

			S.Update(G.Timer.GetElapsedTime()); //The sound engine has to be updated continuously or the sounds stop playing!

		G.Renderer.EndFrame(); //Finished rendering, display the result
	}
	return 0;
}			